from time import sleep

from random import randint
from copy import copy, deepcopy

import socket
import pickle

from threading import Thread


##############################
# Bridge Server
##############################

class Players():
    '''
    info
    '''

    def __init__(self, host='', port=5000):
        self.player_sockets = [False, False, False, False]
        self.player_messages = [None, None, None, None]

        t = Thread(target=self.connection_handler, args=(host, port))
        t.setDaemon(True) # this line makes sure the thread exits when the server is stopped (with ctrl-c)
        t.start()

    def connection_handler(self, host, port):
        server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        server_socket.bind((host, port))
        server_socket.listen(5)

        while True:

            print('Awaiting new connection')
            sock, addr = server_socket.accept()
            print('Connection request from %r. Awaiting player number...' % (addr, ))
            player = int(sock.recv(512))
            print('Player number is %i.  ' % player)
            if self.player_sockets[player]:
                print("There was already a client connected for player %s. That client will be disconnected." % BridgeServer.PLAYERS[player])
                self.player_sockets[player].close()
                self.player_sockets[player] = False

            self.player_sockets[player] = sock
            self.player_sockets[player].settimeout(5)
            print("Player %s connected from %r" % (BridgeServer.PLAYERS[player], addr))

            if self.player_messages[player] is not None:
                self.send(player, self.player_messages[player])


    def send(self, playernr, message):
        self.player_messages[playernr] = message
        if self.player_sockets[playernr] is not False:
            try:
                self.player_sockets[playernr].send(message)
            except socket.error as e:
                print("error in communication with client of player %i " % playernr)
                print("socket error: %s" % e)
                print("removing socket")
                self.player_sockets[playernr].close()
                self.player_sockets[playernr] = False

    def recv(self, playernr):
        while True:
            if self.player_sockets[playernr] is not False:
                try:
                    data = self.player_sockets[playernr].recv(2048)
                    return data
                except socket.timeout:
                    pass
                except socket.error as e:
                    print("error in communication with client of player %i " % playernr)
                    print("socket error: %s" % e)
                    print("removing socket")
                    self.player_sockets[playernr].close()
                    self.player_sockets[playernr] = False
            sleep(1)

class BridgeServer():

    PLAYERS = ['north', 'east', 'south', 'west']
    PLAYERNAMES = ['koen', 'jelle', 'frans', 'pieter']

    CARDS = [ 'SA', 'SH', 'SV', 'SB', 'S10', 'S9', 'S8', 'S7', 'S6', 'S5', 'S4', 'S3', 'S2',
              'HA', 'HH', 'HV', 'HB', 'H10', 'H9', 'H8', 'H7', 'H6', 'H5', 'H4', 'H3', 'H2',
              'RA', 'RH', 'RV', 'RB', 'R10', 'R9', 'R8', 'R7', 'R6', 'R5', 'R4', 'R3', 'R2',
              'KA', 'KH', 'KV', 'KB', 'K10', 'K9', 'K8', 'K7', 'K6', 'K5', 'K4', 'K3', 'K2' ]

    BIDS = [ '0',
             '1K', '1R', '1H', '1S', '1SA',
             '2K', '2R', '2H', '2S', '2SA',
             '3K', '3R', '3H', '3S', '3SA',
             '4K', '4R', '4H', '4S', '4SA',
             '5K', '5R', '5H', '5S', '5SA',
             '6K', '6R', '6H', '6S', '6SA',
             '7K', '7R', '7H', '7S', '7SA' ]


    def __init__(self, host='', port=5000):
        self.players = Players(host=host, port=port)
        print('Game started')
        self.start()

    def _init_gamestate(self):
        self._gamestate = {
            "HIGHEST_BID" : '0',
            "HIGHEST_BIDDER" : -1,
            "CONTRACT" : '',
            "DOUBLET" : False,
            "REDOUBLET" : False,
            "DUMMY" : -1,
            "SCORE" : [0, 0],
            "TABLE" : ['', '', '', ''],
            "ROUNDS" : [],
            "BIDS" : [],
            "HANDS" : ['', '', '', ''],
            "SHOW_DUMMY" : False,
            "SHOW_BIDS" : False,
            "SHOW_ROUNDS" : False,
            "SHOW_ALL" : False,
            "PLAYERS" : self.PLAYERS,
            }

    def start(self):
        start_bidder = 0
        while True:
            valid_contract = False
            while not valid_contract:
                self._init_gamestate()
                FULLHANDS = self.deal()
                self._gamestate['HANDS'] = deepcopy(FULLHANDS)
                valid_contract = self.bid(start_bidder)
            self.play((self._gamestate['DUMMY']-1) % 4)
            self._gamestate['HANDS'] = FULLHANDS
            self._gamestate['TABLE'] = ['', '', '', '']
            self.query(start_bidder, 'end of round, press enter to deal again', show_all=True)
            start_bidder = (start_bidder + 1) % 4

# QUERY

    def query(self, player, message, show_dummy=False, show_all=False):

        self._gamestate['SHOW_DUMMY'] = show_dummy
        self._gamestate['SHOW_ALL'] = show_all

        self._gamestate['message'] = 'no input'
        for i in range(4):
            if i != player:
                self.players.send(i, pickle.dumps(self._gamestate))

        self._gamestate['message'] = message
        self.players.send(player, pickle.dumps(self._gamestate))
        data = self.players.recv(player)

        return data.decode()

# DEAL

    def deal(self):

        DECK = copy(BridgeServer.CARDS)

        HAND0 = []
        HAND1 = []
        HAND2 = []
        HAND3 = []

        for i in range(13):
            HAND0.append(DECK.pop(randint(0,len(DECK)-1)))
            HAND1.append(DECK.pop(randint(0,len(DECK)-1)))
            HAND2.append(DECK.pop(randint(0,len(DECK)-1)))
            HAND3.append(DECK.pop(randint(0,len(DECK)-1)))

        HAND0 = self.sorthand(HAND0)
        HAND1 = self.sorthand(HAND1)
        HAND2 = self.sorthand(HAND2)
        HAND3 = self.sorthand(HAND3)

        return [HAND0, HAND1, HAND2, HAND3]

    def sorthand(self, hand):

        idx = []
        sortedhand = []

        for i in range(13):
            idx.append(BridgeServer.CARDS.index(hand[i]))

        idx.sort()

        for i in range(13):
            sortedhand.append(BridgeServer.CARDS[idx[i]])

        return sortedhand

# BID

    def bid(self, start_bidder):
        bids = []
        bids += [''] * (start_bidder % 4)
        while True:
            for i in range(4):
                player = (i + start_bidder) % 4
                bids.append(self.pickbid(player))
                if bids[-4:] == ['PASS', 'PASS', 'PASS', 'PASS']:
                    self.query(start_bidder, 'no valid bids, press enter to deal again', show_all=True)
                    return False
                elif len(bids) > 3 + (start_bidder % 4) and bids[-3:] == ['PASS', 'PASS', 'PASS']:
                    self._gamestate['BIDS'] = bids
                    self._gamestate['CONTRACT'] = self._gamestate['HIGHEST_BID']
                    #self._gamestate['DUMMY'] = (self._gamestate['HIGHEST_BIDDER'] + 2) % 4
                    if (self._gamestate['HIGHEST_BIDDER'] % 2) == 0:
                        for j in range(0,len(bids),2):
                            if bids[j] != '' and bids[j] != 'PASS' and bids[j][-1] == self._gamestate['CONTRACT'][-1]:
                                self._gamestate['DUMMY'] = ((j+2) % 4)
                                break
                    else:
                        for j in range(1,len(bids),2):
                            if bids[j] != '' and bids[j] != 'PASS' and bids[j][-1] == self._gamestate['CONTRACT'][-1]:
                                self._gamestate['DUMMY'] = ((j+2) % 4)
                                break
                    return True


    def pickbid(self, player):
        bid_valid = False
        message = 'Player %s bid: ' % BridgeServer.PLAYERS[player]
        self._gamestate['TABLE'][player] = 'X'
        while not bid_valid:
            bid = self.query(player, message)
            bid = bid.upper()
            if bid == 'PASS':
                bid_valid = True
            elif BridgeServer.BIDS.count(bid) > 0:
                if BridgeServer.BIDS.index(bid) > BridgeServer.BIDS.index(self._gamestate['HIGHEST_BID']):
                    self._gamestate['HIGHEST_BID'] = bid
                    self._gamestate['HIGHEST_BIDDER'] = player
                    self._gamestate['DOUBLET'] = False
                    self._gamestate['REDOUBLET'] = False
                    bid_valid = True
                else:
                    message = 'bid lower than highest bid, try again: '
            elif bid == 'D':
                if not self._gamestate['DOUBLET'] and self._gamestate['HIGHEST_BID'] != '0':
                    self._gamestate['DOUBLET'] = True
                    bid_valid = True
                elif self._gamestate['DOUBLET']:
                    message = 'alreadry playing DOUBLET, try again: '
                elif self._gamestate['HIGHEST_BID'] == '0':
                    message = 'cannot bid DOUBLET when there is no other bid, try again: '
            elif bid == 'R':
                if self._gamestate['DOUBLET'] and not self._gamestate['REDOUBLET']:
                    self._gamestate['REDOUBLET'] = True
                    bid_valid = True
                elif not self._gamestate['DOUBLET']:
                    message = 'cannot play REDOUBLET when there is no DOUBLET, try again: '
                elif self._gamestate['REDOUBLET']:
                    message = 'already playing REDOUBLET, try again: '
            else:
                message = 'bid not recognised, try again: '
        self._gamestate['TABLE'][player] = bid
        return bid

# PLAY

    def play(self, start_player):
        self._gamestate['SHOW_DUMMY'] = False
        for i in range(13):
            self._gamestate['TABLE'] = ['', '', '', '']
            self._gamestate['FIRST_CARD'] = ''
            for j in range(4):
                player = (j + start_player) % 4
                picked_card = self.pickcard(player)
                if j < 1:
                    self._gamestate['SHOW_DUMMY'] = True
                    self._gamestate['FIRST_CARD'] = picked_card
            self._gamestate['ROUNDS'].append(self._gamestate['TABLE'])
            winning_player = self.pickwin()
            self._gamestate['SCORE'][winning_player % 2] += 1
            start_player = winning_player

    def pickcard(self, player):
        message = 'Player %s pick card: ' % BridgeServer.PLAYERS[player]
        self._gamestate['TABLE'][player] = 'X'
        while True:
            picked_card = self.query(player, message, show_dummy=self._gamestate['SHOW_DUMMY'])
            picked_card = picked_card.upper()
            if self._gamestate['HANDS'][player].count(picked_card) > 0:
                self._gamestate['HANDS'][player].pop(self._gamestate['HANDS'][player].index(picked_card))
                self._gamestate['TABLE'][player] = picked_card
                return picked_card
            else:
                message = 'card does not exist, try again: '

    def pickwin(self):
        idx = [ BridgeServer.CARDS.index(self._gamestate['TABLE'][0]),
                BridgeServer.CARDS.index(self._gamestate['TABLE'][1]),
                BridgeServer.CARDS.index(self._gamestate['TABLE'][2]),
                BridgeServer.CARDS.index(self._gamestate['TABLE'][3]) ]
        for i in range(4):
            if self._gamestate['TABLE'][i][0] != self._gamestate['FIRST_CARD'][0]:
                idx[i] += 100
            if self._gamestate['TABLE'][i][0] == self._gamestate['CONTRACT'][1:]:
                idx[i] -= 200
        m = min(idx)
        wp = idx.index(m)
        self.query(wp, 'Player %s won, hit enter to continue' % BridgeServer.PLAYERS[wp], show_dummy=True)
        return wp






BASIS_PUNTEN =  {'k': 20, 'r': 20, 'h': 30, 's': 30, 'a': 40}
SLAG_PUNTEN =   {'k': 20, 'r': 20, 'h': 30, 's': 30, 'a': 30}
MANCHE_PUNTEN = [ 300, 500 ]
SLEM_PUNTEN =   [ 500, 750 ]
DOWN_PUNTEN = [ [ 100, 200, 200, 300, 300, 300, 300, 300, 300, 300, 300, 300, 300 ],   # niet kwetsbaar
                [ 200, 300, 300, 300, 300, 300, 300, 300, 300, 300, 300, 300, 300 ] ]  # kwetsbaar

# let op! De bonus voor het maken van een ge-redoubleerd contract is 100 punten.
# Dit is anders ten opzichtte van "Bridge, het leerplan voor de complete bridger"
# van Ceest Sint, 1984, waar maar 50 punten worden toegekend.

def bridgecalc(contract, overslagen=0, kwetsbaar=0, doublet=0):
    '''
    contract is een string (bv. '1sa')
    overslagen is 0 voor contract precies gemaakt en negatief voor downslagen, 
    kwetsbaar = 0,1
    doublet = 0,1,2
    '''

    ck = contract[-1].lower()   # contractkleur
    cs = int(contract[0])       # contractslagen
    os = overslagen
    kw = kwetsbaar
    db = doublet

    # contract gehaald
    if os >= 0:

        # basis punten
        pt = (BASIS_PUNTEN[ck] + (cs-1) * SLAG_PUNTEN[ck]) * 2**db

        # manche bonus
        if pt >= 100:
            pt = pt + MANCHE_PUNTEN[kw]
        else:
            pt = pt + 50

        # db bonus
        pt = pt + 50 * db

        # slem bonus
        if cs >= 6:
            pt = pt + SLEM_PUNTEN[kw] * 2**(cs-6)

        # overslagen
        if db == 0:
            pt = pt + os * SLAG_PUNTEN[ck]
        else:
            pt = pt + os * 100 * 2**kw * 2**(db-1)

    # contract niet gehaald
    else:

        if db == 0:
            pt = os * 50 * 2**kw
        else:
            pt = -1 * sum(DOWN_PUNTEN[kw][0:abs(os)]) * 2**(db-1)

    return pt


def table(overslagen=0):

    os = overslagen
    table = []

    Kl = ['k', 'h', 'sa']
    Ks = ['K/R', 'H/S', 'SA ']
    
    print('      |    niet-kwetsbaar     |       kwetsbaar       |')
    print('      |       | Doub. |  Red. |       | Doub. |  Red. |')
    print('-------------------------------------------------------')

    for N in range(1,8):
        for K in range(3):
            row = []
            for W in range(2):
                for D in range(3):
                    if (6 + N + os) > 13 or (6 + N + os) < 0:
                        row.append(0)
                    else:
                        row.append(bridgecalc('%d%s' % (N,Kl[K]), os, W ,D))
            print(' %1d%3s ' % (N,Ks[K]) + '| %5d | %5d | %5d | %5d | %5d | %5d |' % tuple(row))
            table.append(row)




#!/usr/bin/python

helptext = '''
bridge, the network-based bridge card game.
Usage: bridge [OPTIONS] [north/east/south/west]
       bridge [OPTIONS] [koen/jelle/frans/pieter]

OPTIONs:

\t-h, \t--help \t\t this help menu

Start server:
\t-s, \t--startserver \t start the bridgeserver
\t-p, \t--port=port \t port number of bridge server (default 5000)

Start client:
\t-H, \t--host=host \t connect to server host (default localhost)
\t-p, \t--port=port \t connect to server on port port (default 5000)

Example start server:       bridge -s
                            bridge -s -p 5001

Example start client:       bridge north
                            bridge -H hostname -p 5001 north
'''

import sys
import getopt
import socket

try:  # for when nerdbridge is intstalled as package
    from nerdbridge.server import BridgeServer
    from nerdbridge.textclient import BridgeTextClient
except:  # for when nerdbrigde code is simply copied and run from inside folder
    from server import BridgeServer
    from textclient import BridgeTextClient


def run():
    try:
        options, remainder = getopt.getopt(sys.argv[1:],
                'hsp:H:', ['help', 'startserver', 'port=', 'host='])
    except getopt.GetoptError as inst:
        print('Unrecognised commandline options: %s' % inst)
        options = [('--help', '')]
        remainder = []

    startserver = False
    port = 5000
    host = socket.gethostbyname(socket.gethostname())  

    for opt, arg in options:
        if opt in ('-h', '--help'):
            print(helptext)
            sys.exit()
        if opt in ('-s', '--startserver'):
            startserver = True
        if opt in ('-p', '--port'):
            port = int(arg)
        if opt in ('-H', '--host'):
            host = str(arg)

    if startserver:
        print('Starting Bridge Server (IP=%s)' % socket.gethostbyname(socket.gethostname()))
        bs = BridgeServer('', port)
        sys.exit()

    if  len(remainder) == 1 and remainder[0] in BridgeServer.PLAYERS:
        playernr = BridgeServer.PLAYERS.index(remainder[0])
        print('Starting Bridge Client as player "%s"' % remainder[0])
        tc = BridgeTextClient(playernr, host, port)
    elif  len(remainder) == 1 and remainder[0] in BridgeServer.PLAYERNAMES:
        playernr = BridgeServer.PLAYERNAMES.index(remainder[0])
        print('Starting Bridge Client as player "%s"' % remainder[0])
        tc = BridgeTextClient(playernr, host, port)
    else:
        print('to start client, bridge takes at least one argument: [north/east/south/west]')
        print('to start server, run bridge -s')
        print('for more help, run bridge -h')


if __name__ == '__main__':
    run()




import socket
import pickle

##############################
# Bridge Text Client
##############################

class BridgeTextClient():
    '''
    '''

    def __init__(self, player, host='', port=5000):
        self.player = player
        client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        client_socket.connect((host, port))
        client_socket.send(str(player).encode())

        while True:
            data = client_socket.recv(2048)
            self._gamestate = pickle.loads(data)
            data = self.query(self._gamestate['message'])
            if data != ' ':
                client_socket.send(data.encode())


    def query(self, message):
        scr = self.build_screen()
        print(scr)
        if message == 'no input':
            reply = ' '
        else:
            reply = input(message)
            if reply == '':
                reply = 'no reply'
            elif reply.upper() == 'P':
                reply = 'PASS'
        return reply

    def build_hand_block(self, player):
        '''
        build text block of 5 rows and 17 columns containing all cards in a hand
        '''
        if player == -1:
            hand = []
        else:
            hand = self._gamestate['HANDS'][player]
        hand_block = []
        sS = 'S:'
        sH = 'H:'
        sR = 'R:'
        sK = 'K:'
        sS2 = '  '
        sH2 = '  '
        sR2 = '  '
        sK2 = '  '
        iS = 0
        iH = 0
        iR = 0
        iK = 0
        lh = len(hand)
        for i in range(lh):
            if hand[i][0] == 'S':
                if iS <= 6:
                    sS += ' ' + hand[i][1:]
                else:
                    sS2 += ' ' + hand[i][1:]
                iS += 1
            elif hand[i][0] == 'H':
                if iH <= 6:
                    sH += ' ' + hand[i][1:]
                else:
                    sH2 += ' ' + hand[i][1:]
                iH += 1
            elif hand[i][0] == 'R':
                if iR <= 6:
                    sR += ' ' + hand[i][1:]
                else:
                    sR2 += ' ' + hand[i][1:]
                iR += 1
            elif hand[i][0] == 'K':
                if iK <= 6:
                    sK += ' ' + hand[i][1:]
                else:
                    sK2 += ' ' + hand[i][1:]
                iK += 1

        sS = sS + (17 - len(sS)) * ' '
        sH = sH + (17 - len(sH)) * ' '
        sR = sR + (17 - len(sR)) * ' '
        sK = sK + (17 - len(sK)) * ' '

        hand_block.append(sS)
        if len(sS2) > 2:
            sS2 = sS2 + (17 - len(sS2)) * ' '
            hand_block.append(sS2)
        hand_block.append(sH)
        if len(sH2) > 2:
            sH2 = sH2 + (17 - len(sH2)) * ' '
            hand_block.append(sH2)
        hand_block.append(sR)
        if len(sR2) > 2:
            sR2 = sR2 + (17 - len(sR2)) * ' '
            hand_block.append(sR2)
        hand_block.append(sK)
        if len(sK2) > 2:
            sK2 = sK2 + (17 - len(sK2)) * ' '
            hand_block.append(sK2)

        if len(hand_block) == 4:
            hand_block.append(17* ' ')

        return hand_block

    def build_table_block(self):
        '''
        text block of 5 lines and 17 columns
        '''
        table = [ '%10s' % self._gamestate['TABLE'][0] + ' ' * 7,
                 ' ' * 17,
                  '%4s' % self._gamestate['TABLE'][3] + ' ' * 9 + '%4s' % self._gamestate['TABLE'][1],
                 ' ' * 17,
                  '%10s' % self._gamestate['TABLE'][2] + ' ' * 7]
        return table

    def build_bid_block(self):
        '''
        text block of 5+n lines and 17 columns
        '''
        show_bids = self._gamestate['SHOW_BIDS']
        show_all = self._gamestate['SHOW_ALL']
        bidb = []

        if show_bids or show_all:
            bids = self._gamestate['BIDS']
            bids += [''] * ((4 - (len(bids) % 4)) % 4)
            for line in range(len(bids)/4):
                bidb += [ ' %3s %3s %3s %3s ' % (bids[line*4+0][0:3], bids[line*4+1][0:3], bids[line*4+2][0:3], bids[line*4+3][0:3])]

        if len(bidb) < 5:
            bidb += [' ' * 17] * (5 - len(bidb))

        return bidb

    def build_round_block(self):
        '''
        text block of 15 lines and 17 columns
        '''
        show_rounds = self._gamestate['SHOW_ROUNDS']
        show_all = self._gamestate['SHOW_ALL']
        roundb = []

        if show_rounds or show_all:
            rounds = self._gamestate['ROUNDS']
            for line in rounds:
                roundb += [ ' %3s %3s %3s %3s ' % (line[0], line[1], line[2], line[3])]

        roundb += [' ' * 17] * (13 - len(roundb))
        roundb += ['_' * 17] + [' ' * 17]
        return roundb
 
    def build_statusbar(self):
        status = 'Player: %5s    Contract : %3s' % (self._gamestate['PLAYERS'][self.player], self._gamestate['CONTRACT'])
        if self._gamestate['DOUBLET'] and not self._gamestate['REDOUBLET']:
            status += ' (Doublet)'
        if self._gamestate['REDOUBLET']:
            status += ' (Redoublet)'
        status +=  '    Score (NS/EW): %r' % self._gamestate['SCORE']
        return status

    def build_screen(self):
        '''
        builds string block of 77x22 containing full screen
        when bidding rounds larger than
        '''
        player = self.player
        show_dummy = self._gamestate['SHOW_DUMMY']
        show_all = self._gamestate['SHOW_ALL']
        if player == 0 or (self._gamestate['DUMMY'] == 0 and show_dummy) or show_all:
            hand0 = self.build_hand_block(0)
        else:
            hand0 = self.build_hand_block(-1)
        if player == 1 or (self._gamestate['DUMMY'] == 1 and show_dummy) or show_all:
            hand1 = self.build_hand_block(1)
        else:
            hand1 = self.build_hand_block(-1)
        if player == 2 or (self._gamestate['DUMMY'] == 2 and show_dummy) or show_all:
            hand2 = self.build_hand_block(2)
        else:
            hand2 = self.build_hand_block(-1)
        if player == 3 or (self._gamestate['DUMMY'] == 3 and show_dummy) or show_all:
            hand3 = self.build_hand_block(3)
        else:
            hand3 = self.build_hand_block(-1)

        table = self.build_table_block()
        roundb = self.build_round_block()
        bidb = self.build_bid_block()
        status = self.build_statusbar()

        scr = '\n\n\n\n\n'
        scr +=  status + '\n'
        scr += ' ' * 17 + ' | ' + hand0[0] + ' | ' + ' ' * 17 + ' | ' + roundb[0]  + '\n'
        scr += ' ' * 17 + ' | ' + hand0[1] + ' | ' + ' ' * 17 + ' | ' + roundb[1]  + '\n'
        scr += ' ' * 17 + ' | ' + hand0[2] + ' | ' + ' ' * 17 + ' | ' + roundb[2]  + '\n'
        scr += ' ' * 17 + ' | ' + hand0[3] + ' | ' + ' ' * 17 + ' | ' + roundb[3]  + '\n'
        scr += ' ' * 17 + ' | ' + hand0[4] + ' | ' + ' ' * 17 + ' | ' + roundb[4]  + '\n'
        scr += '_' * 57                                       + ' | ' + roundb[5]  + '\n'
        scr += ' ' * 57                                       + ' | ' + roundb[6]  + '\n'
        scr += hand3[0] + ' | ' + table[0] + ' | ' + hand1[0] + ' | ' + roundb[7]  + '\n'
        scr += hand3[1] + ' | ' + table[1] + ' | ' + hand1[1] + ' | ' + roundb[8]  + '\n'
        scr += hand3[2] + ' | ' + table[2] + ' | ' + hand1[2] + ' | ' + roundb[9]  + '\n'
        scr += hand3[3] + ' | ' + table[3] + ' | ' + hand1[3] + ' | ' + roundb[10] + '\n'
        scr += hand3[4] + ' | ' + table[4] + ' | ' + hand1[4] + ' | ' + roundb[11] + '\n'
        scr += '_' * 57                                       + ' | ' + roundb[12] + '\n'
        scr += ' ' * 57                                       + ' | ' + roundb[13] + '\n'
        scr += ' ' * 17 + ' | ' + hand2[0] + ' | ' + ' ' * 17 + ' | ' + roundb[14] + '\n'
        scr += ' ' * 17 + ' | ' + hand2[1] + ' | ' + ' ' * 17 + ' | ' + bidb[0]    + '\n'
        scr += ' ' * 17 + ' | ' + hand2[2] + ' | ' + ' ' * 17 + ' | ' + bidb[1]    + '\n'
        scr += ' ' * 17 + ' | ' + hand2[3] + ' | ' + ' ' * 17 + ' | ' + bidb[2]    + '\n'
        scr += ' ' * 17 + ' | ' + hand0[4] + ' | ' + ' ' * 17 + ' | ' + bidb[3]    + '\n'
        scr += '_' * 77                                                            + '\n'
        scr += ' '
        scr += ' '

        return scr




# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['nerdbridge']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['bridge = nerdbridge.run:run']}

setup_kwargs = {
    'name': 'nerdbridge',
    'version': '0.2.0',
    'description': 'server and client for playing bridge in asci format',
    'long_description': "# nerdbridge\n\nthe nerdbridge package contains a minimal version of a bridge server and client which allows running a (local) bridge server from the command line and connecting 4 clients to it.\nThe clients get an asci style window which shows the hand and allows them to do the bidding and play the game.\n\nThere's also a scorecalc which can calculate the score for each hand after it has been played, but that has not yet been implemented.\n",
    'author': 'Pythor and Keumes',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
